@@ -77,7 +77,6 @@ static GtkWidget *session_command_dialog;
 void apply (void);
 static void help_cb (void);
 static void update_gui (void);
-static void dirty_cb (void);
 static void add_startup_cb (void);
 static void edit_startup_cb (void);
 static void delete_startup_cb (void);
@@ -90,6 +89,25 @@ static void saved_sessions (GtkWidget *widget, GSList *session_names);
 
 #define SESSION_STOCK_EDIT "session-stock-edit"
 
+static void
+selection_changed_cb (GtkTreeSelection *selection, GtkTreeView *view)
+{
+  gboolean sel;
+  GtkWidget *edit_button;
+  GtkWidget *delete_button;
+
+  edit_button = g_object_get_data (G_OBJECT (view), "edit");
+  delete_button = g_object_get_data (G_OBJECT (view), "delete");
+
+  if (!edit_button || !delete_button)
+	return;
+
+  sel = gtk_tree_selection_get_selected (selection, NULL, NULL);
+
+  gtk_widget_set_sensitive (edit_button, sel);
+  gtk_widget_set_sensitive (delete_button, sel);
+}
+
 static void
 register_stock_for_edit (void)
 {
@@ -162,20 +180,41 @@ spc_value_notify (GConfClient *client, guint id, GConfEntry *entry, gpointer tb)
 	}
 }
 
+static gboolean
+show_message_dialog ()
+{
+	GtkWidget *dlg;
+	gint response;
+
+	dlg = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL,
+				      _("Some changes are not saved.\nIs it still OK to exit?"));
+	response = gtk_dialog_run (GTK_DIALOG (dlg));
+	gtk_widget_destroy (dlg);
+	if (response != GTK_RESPONSE_OK) 
+		return TRUE;
+	else
+		return FALSE;
+}
+
+static gboolean
+spc_delete (void)
+{
+	gint ret = FALSE;
+	if (state_dirty)
+		ret = show_message_dialog ();
+
+	if (ret == TRUE) 
+		return TRUE;
+	else {
+		gtk_main_quit ();
+		return FALSE;
+	}
+}
+
 static void
 spc_close (void)
 {
-	if (state_dirty) {
-		GtkWidget *dlg;
-		gint response;
-		dlg = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK_CANCEL,
-					      _("Some changes are not saved.\nIs it still OK to exit?"));
-		response = gtk_dialog_run (GTK_DIALOG (dlg));
-		gtk_widget_destroy (dlg);
-		if (response == GTK_RESPONSE_CANCEL) return;
-	}
-
-	gtk_main_quit ();
+	spc_delete ();
 }
 
 static GtkWidget *
@@ -199,11 +238,15 @@ capplet_build (void)
   dlg = gtk_dialog_new ();
   gtk_window_set_resizable (GTK_WINDOW (dlg), TRUE);
   gtk_window_set_title (GTK_WINDOW (dlg), _("Sessions"));
+  gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+  gtk_container_set_border_width (GTK_CONTAINER (dlg), 5);
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 2);
   help_button = gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
   g_signal_connect (G_OBJECT (help_button), "clicked", help_cb, NULL);
   b = gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gtk_dialog_set_default_response (GTK_DIALOG (dlg), GTK_RESPONSE_CLOSE);
   g_signal_connect (G_OBJECT (b), "clicked", spc_close, NULL);
-  g_signal_connect (dlg, "delete-event", spc_close, NULL);
+  g_signal_connect (dlg, "delete-event", G_CALLBACK(spc_delete), NULL);
 
   /* FIXME: Get rid of global capplet variable, or use only it, not both (Lauris) */
   capplet = dlg;
@@ -214,6 +257,7 @@ capplet_build (void)
   
   /* Set up the notebook */
   notebook = gtk_notebook_new();
+  gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), GTK_POS_TOP);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), notebook, TRUE, TRUE, 0);
 
@@ -275,6 +319,7 @@ capplet_build (void)
 		  		 GTK_WIDGET (sessions_view));
   sessions_sel = gtk_tree_view_get_selection (sessions_view);
   gtk_tree_selection_set_mode (sessions_sel, GTK_SELECTION_SINGLE);
+  g_signal_connect (G_OBJECT (sessions_sel), "changed", (GCallback) selection_changed_cb, sessions_view);
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Session Name"), renderer, "text", 0, NULL);
   gtk_tree_view_append_column (sessions_view, column);
@@ -292,10 +337,14 @@ capplet_build (void)
   button = left_aligned_stock_button (SESSION_STOCK_EDIT);
   gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (edit_session_cb), NULL);
+  gtk_widget_set_sensitive (button, FALSE);
+  g_object_set_data (G_OBJECT (sessions_view), "edit", button);
 
   button = left_aligned_stock_button (GTK_STOCK_DELETE);
   gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (delete_session_cb), NULL);
+  gtk_widget_set_sensitive (button, FALSE);
+  g_object_set_data (G_OBJECT (sessions_view), "delete", button);
  
   /* Set up the hash table to keep tracked of edited session names */
 
@@ -330,7 +379,7 @@ capplet_build (void)
   startup_list = startup_list_read ("Default");
   startup_list_revert = startup_list_duplicate (startup_list);
 
-  vbox = session_properties_create_page (&dirty_cb);
+  vbox = session_properties_create_page (&mark_dirty);
 
   label = gtk_label_new (_("Current Session"));
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
@@ -358,6 +407,7 @@ capplet_build (void)
 		  		 GTK_WIDGET (startup_view));
   startup_sel = gtk_tree_view_get_selection (startup_view);
   gtk_tree_selection_set_mode (startup_sel, GTK_SELECTION_SINGLE);
+  g_signal_connect (G_OBJECT (startup_sel), "changed", (GCallback) selection_changed_cb, startup_view);
   renderer = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Order"), renderer, "text", 1, NULL);
   gtk_tree_view_append_column (startup_view, column);
@@ -379,11 +429,15 @@ capplet_build (void)
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (edit_startup_cb), NULL);
   gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_sensitive (button, FALSE);
+  g_object_set_data (G_OBJECT (startup_view), "edit", button);
   
   button = left_aligned_stock_button (GTK_STOCK_DELETE);
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (delete_startup_cb), NULL);
   gtk_box_pack_start (GTK_BOX (util_vbox), button, FALSE, FALSE, 0);
+  gtk_widget_set_sensitive (button, FALSE);
+  g_object_set_data (G_OBJECT (startup_view), "delete", button);
 
   /* Button for running session-properties */
   a = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
@@ -414,7 +468,7 @@ static void
 help_cb (void)
 {
   GError *error = NULL;
-  gnome_help_display_desktop (NULL, "user-guide", "wgoscustlookandfeel.xml",
+  gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml",
                               "goscustsession-5", &error);
   if (error) {
     GtkWidget *dialog;
@@ -469,8 +523,8 @@ update_gui (void)
 }
 
 /* This is called when an change is made in the client list.  */
-static void
-dirty_cb (void)
+void
+mark_dirty (void)
 {
   state_dirty = TRUE;
   gtk_widget_set_sensitive (apply_button, TRUE);
@@ -484,7 +538,6 @@ add_startup_cb (void)
    * this point so our dialog doesn't die if the user switches
    * away to a different capplet
    */
-  dirty_cb ();
   startup_list_add_dialog (&startup_list, &startup_command_dialog, capplet);
   update_gui ();
 }
@@ -497,7 +550,6 @@ edit_startup_cb (void)
    * this point so our dialog doesn't die if the user switches
    * away to a different capplet
    */
-  dirty_cb ();
   startup_list_edit_dialog (&startup_list, startup_store, startup_sel, &startup_command_dialog, capplet);
   update_gui ();
 }
@@ -506,7 +558,7 @@ edit_startup_cb (void)
 static void
 delete_startup_cb (void)
 {
-  dirty_cb ();
+  mark_dirty ();
   startup_list_delete (&startup_list, startup_store, startup_sel);
   update_gui ();
 }
@@ -561,7 +613,6 @@ add_session_cb(void)
    * this point so our dialog doesn't die if the user switches
    * away to a different capplet
    */
-  dirty_cb ();
   session_list_add_dialog (&session_list, &session_command_dialog, capplet);
   update_gui ();
 }
@@ -576,7 +627,6 @@ edit_session_cb (void)
    * this point so our dialog doesn't die if the user switches
    * away to a different capplet
    */
-  dirty_cb ();
   if (!gtk_tree_selection_get_selected (sessions_sel, NULL, &iter)) return;
   gtk_tree_model_get (sessions_store, &iter, 0, &str, -1);
   session_list_edit_dialog (&session_list, str, &hashed_sessions, &session_command_dialog, capplet);
@@ -589,7 +639,7 @@ delete_session_cb (void)
 {
   GtkTreeIter iter;
   gchar *str;
-  dirty_cb ();
+  mark_dirty ();
   if (!gtk_tree_selection_get_selected (sessions_sel, NULL, &iter)) return;
   gtk_tree_model_get (sessions_store, &iter, 0, &str, -1);
   session_list_delete (&session_list, str, &session_command_dialog);
